@@ -5,7 +5,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -241,6 +241,18 @@ def update_command_single_value(args, options, value):
                 else:
                     j = j+1
 
+    # Special cases at end
+
+    i = 0
+    args_tail = []
+    while i < len(args):
+        if args[i][0] == '$' or args[i] == '&':
+             a = args.pop(i)
+             args_tail.append(a)
+        else:
+             i = i+1
+    args += args_tail
+
     # Return updated list
 
     return args
@@ -360,13 +372,18 @@ def write_export_env(fd, var, value):
     Write the correct command so as to export environment variables.
     """
 
+    if not value:
+        value = ''
     if sys.platform.startswith('win'):
         export_cmd = 'set ' + var + '=' + value
     else:
         if get_shell_type()[-3:] == 'csh': # handle C-type shells
             export_cmd = 'setenv ' + var + ' ' + value
         else:                              # handle Bourne-type shells
-            export_cmd = 'export ' + var + '=' + value
+            if value:
+                export_cmd = 'export ' + var + '=' + value
+            else:
+                export_cmd = 'unset ' + var
     export_cmd = export_cmd + '\n'
     fd.write(export_cmd)
 
@@ -470,17 +487,17 @@ def get_script_return_code():
 
 #-------------------------------------------------------------------------------
 
-def run_command(cmd, pkg = None, echo = False,
+def run_command(args, pkg = None, echo = False,
                 stdout = sys.stdout, stderr = sys.stderr, env = None):
     """
     Run a command.
     """
     if echo == True:
-        if type(cmd) == str:
-            stdout.write(str(cmd) + '\n')
+        if type(args) == str:
+            stdout.write(str(args) + '\n')
         else:
             l = ''
-            for s in cmd:
+            for s in args:
                 if (s.find(' ') > -1):
                     l += ' ' + enquote_arg(s)
                 else:
@@ -508,17 +525,30 @@ def run_command(cmd, pkg = None, echo = False,
     if (stderr != sys.stderr):
         kwargs['stderr'] = stderr
 
-    if type(cmd) == str:
-        p = subprocess.Popen(cmd,
-                             shell=True,
-                             executable=get_shell_type(),
-                             universal_newlines=True,
-                             env = env,
-                             **kwargs)
-    else:
-        p = subprocess.Popen(cmd, universal_newlines=True, env = env, **kwargs)
-
-    p.communicate()
+    returncode = 1
+    try:
+        if type(args) == str:
+            p = subprocess.Popen(args,
+                                 shell=True,
+                                 executable=get_shell_type(),
+                                 universal_newlines=True,
+                                 env = env,
+                                 **kwargs)
+        else:
+            p = subprocess.Popen(args, universal_newlines=True, env = env, **kwargs)
+        p.communicate()
+        returncode = p.returncode
+    except Exception as e:
+        import traceback
+        exc_type, exc_value, exc_traceback = sys.exc_info()
+        traceback.print_exception(exc_type, exc_value, exc_traceback,
+                                  limit=2, file=sys.stderr)
+        print("")
+        print("Failed calling subprocess with:")
+        print("  args = " + str(args))
+        print("  env = " + str(env))
+        print("  kwargs = " + str(kwargs))
+        sys.exit(1)
 
     # Reset the PATH to its previous value
 
@@ -526,7 +556,7 @@ def run_command(cmd, pkg = None, echo = False,
         if pkg.config.features['relocatable'] == "yes":
             os.environ['PATH'] = saved_path
 
-    return p.returncode
+    return returncode
 
 #-------------------------------------------------------------------------------
 
@@ -672,21 +702,44 @@ def source_syrthes_env(pkg):
     # Determine SYRTHES home
 
     syrthes_home = None
-    env_syrthes_home = os.getenv('SYRTHES4_HOME')
 
+    # Home based on configuration info
+
+    cfg_syrthes_home = None
     config = configparser.ConfigParser()
     config.read(pkg.get_configfiles())
     if config.has_option('install', 'syrthes'):
-        syrthes_home = os.path.join(config.get('install', 'syrthes'))
-        if not os.path.isdir(syrthes_home):
+        cfg_syrthes_home = os.path.join(config.get('install', 'syrthes'))
+        if not os.path.isdir(cfg_syrthes_home):
             sys.stderr.write("\nIncorrect install/syrthes entry "
                          + "specified in one of:\n")
             for f in pkg.get_configfiles():
                 sys.stderr.write("  " + f + "\n")
             sys.stderr.write("Directory: '"
-                             + syrthes_home + "' does not exist.\n")
+                             + cfg_syrthes_home + "' does not exist.\n")
             sys.exit(1)
-    else:
+            cfg_syrthes_home = None
+        else:
+            syrthes_home = cfg_syrthes_home
+
+    # Home based on current environment
+
+    # Try to base Syrthes version on path used to import the syrthes module,
+    # for consistency with case creation parameters; it this is not enough,
+    # use existing environment or load one based on current configuration.
+
+    try:
+        for p in sys.path:
+            if p[-14:] == '/share/syrthes' or p[-14:] == '\share\syrthes':
+                syr_profile = os.path.join(p[:,-14], 'bin', 'syrthes.profile')
+                if os.path.isfile(syr_profile):
+                    source_shell_script(syr_profile)
+    except Exception:
+        pass
+
+    env_syrthes_home = os.getenv('SYRTHES4_HOME')
+
+    if not syrthes_home:
         syrthes_home = env_syrthes_home
 
     if not syrthes_home:
@@ -824,7 +877,9 @@ class batch_info:
             rtime = get_command_output(cmd)
         elif self.batch_type == 'SLURM':
             cmd = "squeue -h -j $SLURM_JOBID -o %L"
-            rtime = cs_batch.parse_wall_time_slurm(get_command_output(cmd))
+            # In case of job array, check on first line
+            rs = get_command_output(cmd).splitlines()[0]
+            rtime = cs_batch.parse_wall_time_slurm(rs)
 
         return rtime
 
@@ -846,6 +901,8 @@ class resource_info(batch_info):
         self.n_procs = None
         self.n_nodes = None
         self.n_threads = None
+        self.n_procs_ri = None
+        self.n_threads_ri = None
 
         # Pre-set the number of threads if OMP_NUM_THREADS given
 
@@ -985,6 +1042,7 @@ class resource_info(batch_info):
             else:
                 s = os.getenv('PE_HOSTFILE')
                 if s != None:
+                    self.manager = 'SGE'
                     self.hosts_file = '$PE_HOSTFILE'
 
         # Check hosts file presence
@@ -1007,16 +1065,21 @@ class resource_info(batch_info):
             elif self.hosts_list != None:
                 self.n_procs_from_hosts_list(self.hosts_list)
 
+        # keep track of automatically determined values
+
+        self.n_procs_ri = self.n_procs
+        self.n_threads_ri = self.n_threads
+
         # Check and possibly set number of processes
 
         if n_procs != None:
             if self.n_procs != None:
                 if self.n_procs != n_procs:
                     sys.stderr.write('Warning:\n'
-                                     +'   Will use ' + str(self.n_procs)
+                                     +'   Will try to use ' + str(n_procs)
                                      + ' processes while resource manager ('
                                      + self.manager + ')\n   allows for '
-                                     + str(n_procs) + '.\n\n')
+                                     + str(self.n_procs) + '.\n\n')
             self.n_procs = n_procs
 
         if self.n_procs == None:
@@ -1027,7 +1090,7 @@ class resource_info(batch_info):
         if n_threads != None:
             if self.n_threads != None and n_threads != self.n_threads:
                 sys.stderr.write('Warning:\n'
-                                 +'   Will use ' + str(self.n_threads)
+                                 +'   Will try to use ' + str(self.n_threads)
                                  + ' threads per task while resource manager ('
                                  + self.manager + ')\n   allows for '
                                  + str(n_threads) + '.\n\n')
@@ -1112,58 +1175,8 @@ class resource_info(batch_info):
         # Check for resource manager and eventual hostsfile
 
         elif self.manager == 'SLURM':
-            s = os.getenv('SLURM_TASKS_PER_NODE')
-            hosts_count = None
-            if s != None:
-                # Syntax may be similar to SLURM_TASKS_PER_NODE=2(x3),1"
-                # indicating three nodes will each execute 2 tasks and
-                # the  fourth node will execute 1 task.
-                hosts_count = []
-                for s0 in s.split(','):
-                    i = s0.find('(')
-                    if i > -1:
-                        for j in range(int(s0[i+2:-1])):
-                            hosts_count.append(int(s0[0:i]))
-                    else:
-                        hosts_count.append(int(s0))
-            s = os.getenv('SLURM_NODELIST')
-            if s != None:
-                hosts_list = []
-                l = 0
-                # List uses a compact representation
-                for s0 in s.split(','):
-                    i = s0.find('[')
-                    if i > -1:
-                        basename = s0[0:i]
-                        for s1 in s0[i+1:-1].split(','):
-                            s2 = s1.split('-')
-                            if len(s2) > 1:
-                                fmt_s = basename + '%0' + str(len(s2[0])) + 'd'
-                                for j in range(int(s2[0]), int(s2[1])+1):
-                                    host_name = fmt_s % j
-                                    if hosts_count:
-                                        for k in range(hosts_count[l]):
-                                            hosts_list.append(host_name)
-                                        l += 1
-                                    else:
-                                        hosts_list.append(host_name)
-                            else:
-                                if hosts_count:
-                                    for k in range(hosts_count[l]):
-                                        hosts_list.append(s2[0])
-                                    l += 1
-                                else:
-                                    hosts_list.append(s2[0])
-                    else:
-                        if hosts_count:
-                            for k in range(hosts_count[l]):
-                                hosts_list.append(s0)
-                            l += 1
-                        else:
-                            hosts_list.append(s0)
-            else:
-                hosts_list = get_command_output('srun hostname -s').split()
-                hosts_list.sort()
+            hosts_list = get_command_output('srun hostname -s').split()
+            hosts_list.sort()
 
         elif self.manager == 'LSF':
             s = os.getenv('LSB_MCPU_HOSTS')
@@ -1292,9 +1305,13 @@ class mpi_environment:
         # We may have a specific syntax for tasks per node if
         # a launcher from a resource manager is used:
 
-        if resource_info and self.mpiexec and not self.mpiexec_n_per_node:
+        if resource_info and self.mpiexec:
             if os.path.basename(self.mpiexec)[:4] == 'srun':
-                self.mpiexec_n_per_node = ' --ntasks-per-node='
+                if resource_info.n_procs_ri != None \
+                  and resource_info.n_procs_ri != resource_info.n_procs:
+                    self.mpiexec_n = ' --ntasks='
+                elif not self.mpiexec_n_per_node:
+                    self.mpiexec_n_per_node = ' --ntasks-per-node='
 
         # Initialize based on known MPI types, or default.
 
